#!/usr/bin/env python
# -*- coding: utf-8 -*-

__version__ = '1.0'

from .make_settings import settings
from .utils import LocalProxy
from .app import TorextApp


current_app = LocalProxy(lambda: TorextApp.current_app)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pprint
import logging
import datetime
import traceback
from gevent import pywsgi


class FormattedWSGIHandler(pywsgi.WSGIHandler):
    logger = logging.getLogger('gevent.wsgi')

    status_level_map = {
        200: logging.INFO,
        300: logging.INFO,
        400: logging.WARNING,
        500: logging.ERROR
    }

    def format_request(self):
        """Override for better log format

        Tornado format:
        [INFO 2015-03-24 11:29:57 app:521] 200 GET /static/css/lib/pure-min.css (127.0.0.1) 6.76ms
        Current format:
        [gevent.wsgi] INFO 127.0.0.1 - - [2015-03-24 11:18:45] "GET /test HTTP/1.1" 200 304 0.000793
        """
        fmt = '{now} {status} {requestline} ({client_address}) {response_length} {delta}ms'

        requestline = getattr(self, 'requestline')
        if requestline:
            # Original "GET / HTTP/1.1", remove the "HTTP/1.1"
            requestline = ' '.join(requestline.split(' ')[:-1])
        else:
            requestline = '???'

        if self.time_finish:
            delta = '%.2f' % ((self.time_finish - self.time_start) * 1000)
        else:
            delta = '-'

        data = dict(
            now=datetime.datetime.now().replace(microsecond=0),
            response_length=self.response_length or '-',
            client_address=self.client_address[0] if isinstance(self.client_address, tuple) else self.client_address,
            status=str(self._get_status_int()),
            requestline=requestline,
            delta=delta,
        )

        return fmt.format(**data)

    def _get_status_int(self):
        if not hasattr(self, '_status_int'):
            try:
                self._status_int = int(getattr(self, 'status', '000').split()[0])
            except ValueError:
                self._status_int = 0
        return self._status_int

    def log_request(self):
        if not self.server.log_enabled:
            return

        msg = self.format_request()

        status = self._get_status_int()

        if status < 300:
            flag = 200
        elif status < 400:
            flag = 300
        elif status < 500:
            flag = 400
        else:
            flag = 500
        level = self.status_level_map[flag]

        self.logger.log(level, msg)

    def handle_error(self, type_, value, tb):
        """This method copies the code from pywsgi.WSGIHandler.handle_error,
        change the write part to be a reflection of traceback and environ
        """
        if not issubclass(type_, pywsgi.GreenletExit):
            self.server.loop.handle_error(self.environ, type_, value, tb)
        if self.response_length:
            self.close_connection = True
        else:
            tb_stream = traceback.format_exception(type_, value, tb)
            del tb
            tb_stream.append('\n')
            tb_stream.append(pprint.pformat(self.environ))
            body = ''.join(tb_stream)
            headers = pywsgi._INTERNAL_ERROR_HEADERS[:]
            headers[2] = ('Content-Length', str(len(body)))

            self.start_response(pywsgi._INTERNAL_ERROR_STATUS, headers)
            self.write(body)


class WSGIServer(pywsgi.WSGIServer):
    handler_class = FormattedWSGIHandler

    def __init__(self, *args, **kwargs):
        if 'log_enabled' in kwargs:
            self.log_enabled = kwargs.pop('log_enabled')
        else:
            self.log_enabled = True
        super(WSGIServer, self).__init__(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class TorextException(Exception):
    def __init__(self, message=''):
        if isinstance(message, str):
            message = message.decode('utf8')
        self.message = message

    def __str__(self):
        return unicode(self).encode('utf-8')

    def __unicode__(self):
        return self.message


# route.py

class URLRouteError(TorextException):
    """error in router"""


# make_settings.py

class SettingsError(TorextException):
    pass


# app.py

class ArgsParseError(TorextException):
    pass


# script.py

class CommandArgumentError(TorextException):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Features:
    * scope control for session in tornado environment
    * signal in commit process (use new api: orm.events)
    * signal in connection process

Finished:
    * bind all frequently-used classes & methods to SQLAlchemy object
    * Model class for creating new models
    * customizable Query object
"""

import os
import json
import logging
import functools
import sqlalchemy
import sqlalchemy.orm
from sqlalchemy import func
from sqlalchemy.engine.url import make_url
from sqlalchemy.orm import sessionmaker, scoped_session, Query, load_only
from sqlalchemy.ext.declarative import declarative_base, declared_attr
from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound
from sqlalchemy.ext.declarative.api import DeclarativeMeta
from sqlalchemy.ext.mutable import Mutable

from torext.app import TorextApp
from torext.errors import TorextException


# TODO refine
logger = logging.getLogger('sqlalchemy')
logger.propagate = 0


class DoesNotExist(TorextException):
    pass


class MultipleObjectsReturned(TorextException):
    pass


def _make_table(db):
    def _Table(*args, **kwargs):
        """
        if passing (name, column, ..), existing metadata will be added to args,
        return Table(name, metadata, column, ..)
        """
        if len(args) > 1 and isinstance(args[1], db.Column):
            args = (args[0], db.metadata) + args[1:]
        return sqlalchemy.Table(*args, **kwargs)
    return _Table


def _set_default_query_class(d):
    if 'query_class' not in d:
        d['query_class'] = BaseQuery


def _wrap_with_default_query_class(fn):
    @functools.wraps(fn)
    def newfn(*args, **kwargs):
        _set_default_query_class(kwargs)
        if "backref" in kwargs:
            backref = kwargs['backref']
            if isinstance(backref, basestring):
                backref = (backref, {})
            _set_default_query_class(backref[1])
        return fn(*args, **kwargs)
    return newfn


class BaseQuery(Query):
    """
    Add any function on this class, so that it can be called on SomeModel.query
    """
    @property
    def entity_class(self):
        return self._entities[0].entities[0].class_

    def _raise_not_exist(self, query_repr, message=None):
        if not message:
            message = '{} does not exist: {}'.format(self.entity_class.__name__, query_repr)
        raise self.entity_class.DoesNotExist(message)

    def _raise_multiple_results(self, query_repr):
        message = '{} got multiple objects: {}'.format(self.entity_class.__name__, query_repr)
        raise MultipleObjectsReturned(message)

    def get_or_raise(self, id, message=None):
        rv = self.get(id)
        if rv is None:
            self._raise_not_exist('id=%s' % id, message)
        return rv

    def one_or_raise(self, message=None):
        try:
            rv = self.one()
        except NoResultFound:
            self._raise_not_exist(self.statement, message)
        except MultipleResultsFound:
            self._raise_multiple_results(self.statement)
        return rv

    def paginator(self, _max, index):
        count = self.with_entities(func.count(self.entity_class.id)).scalar()

        if (index - 1) * _max > count or index <= 0:
            raise ValueError('index or max argument overflow')

        if not count:
            return count, self

        ret = self.offset(_max * (index - 1)).limit(_max)
        return count, ret

    def load_only(self, *args, **kwargs):
        return self.options(load_only(*args, **kwargs))


class TorextDeclarativeMeta(DeclarativeMeta):
    def __new__(cls, name, bases, attrs):
        print 'cls cls', cls
        model_cls = super(TorextDeclarativeMeta, cls).__new__(cls, name, bases, attrs)
        # model_cls = type.__new__(cls, name, bases, attrs)

        exccls_name = '%sDoesNotExist' % name
        exccls_attrs = {'__module__': model_cls.__module__}

        model_cls.DoesNotExist = type(exccls_name, (DoesNotExist, ), exccls_attrs)
        return model_cls


class _Model(object):
    query = None

    @declared_attr
    def __tablename__(cls):
        return cls.__name__.lower()

    def __init__(self, **kwargs):
        for k, v in kwargs.iteritems():
            setattr(self, k, v)


class SQLAlchemy(object):
    """
    Usage:

    1. with app
    # database.py
    >>> from torext.sql import SQLAlchemy
    >>> db = SQLAlchemy()

    # app.py
    >>> from myproject.database import db
    >>> db.init_app(app)

    2. without app
    >>> from torext.sql import SQLAlchemy
    >>> db = SQLAlchemy('sqlite://')
    """

    def __init__(self, app=None, uri=None, config=None, session_options=None):
        self._engine = None

        # session is created without an engine,
        # it will be binded with engine later by uri argument
        # or through app settings comes from `init_app` invoking
        self.session = self.create_scoped_session(session_options)

        self.Model = self.make_declarative_base()

        # config
        if app:
            assert isinstance(app, TorextApp), 'app should be an instance of TorextApp'
            self.config = self.init_config(app.settings['SQLALCHEMY'])
        else:
            self.config = self.init_config(config)
        if uri:
            self.config['uri'] = uri

        if self.config['uri']:
            self.session.configure(bind=self.engine)

        self.app = app
        self._include_sqlalchemy()

    def init_app(self, app):
        self.app = app
        self.config = self.init_config(app.settings['SQLALCHEMY'])
        self.session.configure(bind=self.engine)

    def create_scoped_session(self, options=None):
        if options is None:
            options = {}
        return scoped_session(sessionmaker(query_cls=BaseQuery), **options)

    def make_declarative_base(self):
        base = declarative_base(cls=_Model, name='Model', metaclass=TorextDeclarativeMeta)
        base.query = self.session.query_property()
        return base

    def _include_sqlalchemy(self):
        for module in sqlalchemy, sqlalchemy.orm:
            for key in module.__all__:
                if not hasattr(self, key):
                    setattr(self, key, getattr(module, key))
        # Note: self.Table does not attempt to be a SQLAlchemy Table class.
        self.Table = _make_table(self)
        self.relationship = _wrap_with_default_query_class(self.relationship)
        self.relation = _wrap_with_default_query_class(self.relation)
        self.dynamic_loader = _wrap_with_default_query_class(self.dynamic_loader)

    def init_config(self, incoming=None):
        config = {
            'uri': None,
            'pool_size': None,
            'pool_timeout': None,
            'pool_recycle': None,
            'echo': False,
        }
        if incoming:
            for k in config:
                if k in incoming:
                    config[k] = incoming[k]
        return config

    @property
    def engine(self):
        if not self._engine:
            self._engine = self.get_engine()
        return self._engine

    def get_engine(self):
        assert self.config['uri'], 'uri should not be empty,'\
            ' assign by call init_app or explicitly passing'
        uri_obj = make_url(self.config['uri'])
        options = {
            'convert_unicode': True,
            'echo': self.config.get('echo', False)
        }
        for k in ('pool_size', 'pool_timeout', 'pool_recycle'):
            v = self.config.get(k, None)
            if v is not None:
                options[k] = v
        self._apply_driver_hacks(uri_obj, options)
        logging.info('SQLAlchemy engine config: %s, %s', uri_obj, options)
        engine = sqlalchemy.create_engine(uri_obj, **options)
        return engine

    def _apply_driver_hacks(self, uri_obj, options):
        if uri_obj.drivername == 'mysql':
            uri_obj.query.setdefault('charset', 'utf8')
            options.setdefault('pool_size', 10)
            options.setdefault('pool_recycle', 7200)
        elif uri_obj.drivername == 'sqlite':
            pool_size = options.get('pool_size')
            in_memory = False
            # we go to memory and the pool size was explicitly set to 0
            # which is fail.  Let the user know that
            if uri_obj.database in (None, '', ':memory:'):
                in_memory = True
                if pool_size == 0:
                    raise RuntimeError('SQLite in memory database with an '
                                       'empty queue not possible due to data '
                                       'loss.')
            # if pool size is None or explicitly set to 0 we assume the
            # user did not want a queue for this sqlite connection and
            # hook in the null pool.
            elif not pool_size:
                from sqlalchemy.pool import NullPool
                options['poolclass'] = NullPool

            # if it's not an in memory database we make the path absolute.
            if not in_memory and self.app:
                uri_obj.database = os.path.join(self.app.root_path, uri_obj.database)

    def _execute_operation(self, operation):
            op = getattr(self.Model.metadata, operation)
            op(self.engine)

    def create_all(self):
        self._execute_operation('create_all')

    def drop_all(self):
        self._execute_operation('drop_all')

    def reflect(self):
        self._execute_operation('reflect')

    def __repr__(self):
        return '<%s engine=%r>' % (
            self.__class__.__name__,
            self.config.get('uri')
        )


# helpers

class MutationObj(Mutable):
    @classmethod
    def coerce(cls, key, value):
        if isinstance(value, dict) and not isinstance(value, MutationDict):
            return MutationDict.coerce(key, value)
        if isinstance(value, list) and not isinstance(value, MutationList):
            return MutationList.coerce(key, value)
        return value

    @classmethod
    def _listen_on_attribute(cls, attribute, coerce, parent_cls):
        key = attribute.key
        if parent_cls is not attribute.class_:
            return

        # rely on "propagate" here
        parent_cls = attribute.class_

        def load(state, *args):
            val = state.dict.get(key, None)
            if coerce:
                val = cls.coerce(key, val)
                state.dict[key] = val
            if isinstance(val, cls):
                val._parents[state.obj()] = key

        def set(target, value, oldvalue, initiator):
            if not isinstance(value, cls):
                value = cls.coerce(key, value)
            if isinstance(value, cls):
                value._parents[target.obj()] = key
            if isinstance(oldvalue, cls):
                oldvalue._parents.pop(target.obj(), None)
            return value

        def pickle(state, state_dict):
            val = state.dict.get(key, None)
            if isinstance(val, cls):
                if 'ext.mutable.values' not in state_dict:
                    state_dict['ext.mutable.values'] = []
                state_dict['ext.mutable.values'].append(val)

        def unpickle(state, state_dict):
            if 'ext.mutable.values' in state_dict:
                for val in state_dict['ext.mutable.values']:
                    val._parents[state.obj()] = key

        sqlalchemy.event.listen(parent_cls, 'load', load, raw=True, propagate=True)
        sqlalchemy.event.listen(parent_cls, 'refresh', load, raw=True, propagate=True)
        sqlalchemy.event.listen(attribute, 'set', set, raw=True, retval=True, propagate=True)
        sqlalchemy.event.listen(parent_cls, 'pickle', pickle, raw=True, propagate=True)
        sqlalchemy.event.listen(parent_cls, 'unpickle', unpickle, raw=True, propagate=True)


class MutationDict(MutationObj, dict):
    @classmethod
    def coerce(cls, key, value):
        """Convert plain dictionary to MutationDict"""
        self = MutationDict((k,MutationObj.coerce(key,v)) for (k,v) in value.items())
        self._key = key
        return self

    def __setitem__(self, key, value):
        dict.__setitem__(self, key, MutationObj.coerce(self._key, value))
        self.changed()

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        self.changed()


class MutationList(MutationObj, list):
    @classmethod
    def coerce(cls, key, value):
        """Convert plain list to MutationList"""
        self = MutationList((MutationObj.coerce(key, v) for v in value))
        self._key = key
        return self

    def __setitem__(self, idx, value):
        list.__setitem__(self, idx, MutationObj.coerce(self._key, value))
        self.changed()

    def __setslice__(self, start, stop, values):
        list.__setslice__(self, start, stop, (MutationObj.coerce(self._key, v) for v in values))
        self.changed()

    def __delitem__(self, idx):
        list.__delitem__(self, idx)
        self.changed()

    def __delslice__(self, start, stop):
        list.__delslice__(self, start, stop)
        self.changed()

    def append(self, value):
        list.append(self, MutationObj.coerce(self._key, value))
        self.changed()

    def insert(self, idx, value):
        list.insert(self, idx, MutationObj.coerce(self._key, value))
        self.changed()

    def extend(self, values):
        list.extend(self, (MutationObj.coerce(self._key, v) for v in values))
        self.changed()

    def pop(self, *args, **kw):
        value = list.pop(self, *args, **kw)
        self.changed()
        return value

    def remove(self, value):
        list.remove(self, value)
        self.changed()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    import sqlalchemy
except ImportError:
    print 'sqlalchemy is not installed, skip testing'
    from nose.plugins.skip import SkipTest
    raise SkipTest

try:
    with open('.mysqluri', 'r') as f:
        MYSQL_URI = f.read().strip()
except IOError:
    print 'mysql is not configured, skip sql_test'
    from nose.plugins.skip import SkipTest
    raise SkipTest

from nose.tools import assert_raises
from torext.app import TorextApp
from torext.sql import SQLAlchemy
from torext import errors


class TestSQLModule(object):
    def setup(self):
        app = TorextApp()
        app.settings['DEBUG'] = False
        app.settings['SQLALCHEMY'] = {
            'uri': MYSQL_URI,
            'echo': True
        }
        app.setup()

        self.create_database()

        db = SQLAlchemy(app=app)

        class User(db.Model):
            id = db.Column(db.Integer, primary_key=True)
            name = db.Column(db.String(20))

        class Hub(db.Model):
            id = db.Column(db.Integer, primary_key=True)

        self.User = User
        self.Hub = Hub

        db.create_all()

        self.app = app
        self.db = db

    def teardown(self):
        # clean possible opened transactions
        self.db.session.commit()

        self.drop_database()

    def create_database(self):
        splited = MYSQL_URI.split('/')
        self.database_name = splited[-1]
        no_db_uri = '/'.join(splited[:-1])
        self.no_db_engine = sqlalchemy.create_engine(no_db_uri)
        self.no_db_engine.execute('create database if not exists %s;' % self.database_name)

    def drop_database(self):
        self.no_db_engine.execute('drop database %s;' % self.database_name)

    def test_create_delete(self):
        u = self.User(name='reorx')
        self.db.session.add(u)
        self.db.session.commit()

        us = self.User.query.all()
        assert len(us) == 1

        self.db.session.delete(u)

        us = self.User.query.all()
        assert len(us) == 0

    def test_get_or_raise(self):
        u = self.User(name='reorx')
        self.db.session.add(u)
        self.db.session.commit()

        assert self.User.query.get_or_raise(1).name == 'reorx'

        print 'DoesNotExist class', self.User.DoesNotExist
        with assert_raises(self.User.DoesNotExist):
            self.User.query.get_or_raise(2)

    def test_one_or_raise(self):
        u = self.User(name='reorx')
        self.db.session.add(u)

        u1 = self.User(name='reorx')
        self.db.session.add(u1)

        self.db.session.commit()

        assert self.User.query.filter(self.User.id == 2).one_or_raise().name == 'reorx'

        with assert_raises(self.User.DoesNotExist):
            try:
                self.User.query.filter(self.User.name == 'others').one_or_raise()
            except self.Hub.DoesNotExist:
                pass

        with assert_raises(errors.MultipleObjectsReturned):
            self.User.query.filter(self.User.name == 'reorx').one_or_raise()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import datetime
import functools


import json

try:
    from tornado.util import raise_exc_info
except ImportError:
    def raise_exc_info(exc_info):
        """Re-raise an exception (with original traceback) from an exc_info tuple.

        The argument is a ``(type, value, traceback)`` tuple as returned by
        `sys.exc_info`.
        """
        # 2to3 isn't smart enough to convert three-argument raise
        # statements correctly in some cases.
        if isinstance(exc_info[1], exc_info[0]):
            raise exc_info[1], None, exc_info[2]
            # After 2to3: raise exc_info[1].with_traceback(exc_info[2])
        else:
            # I think this branch is only taken for string exceptions,
            # which were removed in Python 2.6.
            raise exc_info[0], exc_info[1], exc_info[2]
            # After 2to3: raise exc_info[0](exc_info[1]).with_traceback(exc_info[2])


def generate_cookie_secret():
    import uuid
    import base64
    return base64.b64encode(uuid.uuid4().bytes + uuid.uuid4().bytes)


json_decode = functools.partial(json.loads, encoding='utf-8')


json_encode = functools.partial(json.dumps, ensure_ascii=False)


def timesince(t):
    if not isinstance(t, datetime.datetime):
        raise TypeError('Time should be instance of datetime.datetime')
    now = datetime.datetime.utcnow()
    delta = now - t
    if not delta.days:
        if delta.seconds / 3600:
            return '{0} hours ago'.format(delta.seconds / 3600)
        return '{0} minutes ago'.format(delta.seconds / 60)
    if delta.days / 365:
        return '{0} years ago'.format(delta.days / 365)
    if delta.days / 30:
        return '{0} months ago'.format(delta.days / 30)
    return '{0} days ago'.format(delta.days)


def pprint(o):
    import pprint
    pprinter = pprint.PrettyPrinter(indent=4)
    pprinter.pprint(o)


class SingletonMixin(object):
    """Globally hold one instance class

    Usage:
    >>> class SpecObject(SingletonMixin):
    ...     pass

    >>> ins = SpecObject.instance()
    """
    @classmethod
    def instance(cls, *args, **kwgs):
        """Will be the only instance"""
        if not hasattr(cls, "_instance"):
            cls._instance = cls(*args, **kwgs)
        return cls._instance


def split_kwargs(kwgs_tuple, kwgs):
    _kwgs = {}
    for i in kwgs_tuple:
        if i in kwgs:
            _kwgs[i] = kwgs.pop(i)
    return _kwgs


class ObjectDict(dict):
    """
    retrieve value of dict in dot style
    """
    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError('Has no attribute %s' % key)

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, key):
        try:
            del self[key]
        except KeyError:
            raise AttributeError(key)

    def __str__(self):
        return '<ObjectDict %s >' % dict(self)


# def import_underpath_module(path, name):
#     """
#     arguments::
#     :name :: note that name do not contain `.py` at the end
#     """
#     importer = pkgutil.get_importer(path)
#     logging.debug('loading handler module: ' + name)
#     return importer.find_module(name).load_module(name)
#
#
# def autoload_submodules(dirpath):
#     """Load submodules by dirpath
#     NOTE. ignore packages
#     """
#     import pkgutil
#     importer = pkgutil.get_importer(dirpath)
#     return (importer.find_module(name).load_module(name)
#             for name, is_pkg in importer.iter_modules())


######################################
# borrow from django.utils.importlib #
######################################

# Taken from Python 2.7 with permission from/by the original author.

def _resolve_name(name, package, level):
    """Return the absolute name of the module to be imported."""
    if not hasattr(package, 'rindex'):
        raise ValueError("'package' not set to a string")
    dot = len(package)
    for x in xrange(level, 1, -1):
        try:
            dot = package.rindex('.', 0, dot)
        except ValueError:
            raise ValueError("attempted relative import beyond top-level\
                package")
    return "%s.%s" % (package[:dot], name)


def import_module(name, package=None):
    """Import a module.

    The 'package' argument is required when performing a relative import. It
    specifies the package to use as the anchor point from which to resolve the
    relative import to an absolute import.

    Usage:
    >>> epoll_module = import_module('tornado.platform.epoll')
    """
    if name.startswith('.'):
        if not package:
            raise TypeError("relative imports require the 'package' argument")
        level = 0
        for character in name:
            if character != '.':
                break
            level += 1
        name = _resolve_name(name[level:], package, level)
    __import__(name)
    return sys.modules[name]


_abspath = lambda x: os.path.abspath(x)
_join = lambda x, y: os.path.join(x, y)


def add_to_syspath(pth, relative_to=None):
    if relative_to:
        pth = _join(relative_to, pth)
    if _abspath(pth) in [_abspath(i) for i in sys.path]:
        print 'path %s is in sys.path, pass' % pth
    else:
        print 'add path %s to sys.path' % pth
        sys.path.insert(0, pth)


def start_shell(extra_vars=None):
    import code
    import rlcompleter
    import readline

    class CustomCompleter(rlcompleter.Completer):
        def complete(self, text, state):
            if text == "":
                # you could  replace \t to 4 or 8 spaces if you prefer indent via spaces
                return ['    ', None][state]
            else:
                # return rlcompleter.Completer.complete(self, text, state)
                return super(CustomCompleter, self).complete(text, state)

    readline.parse_and_bind("tab: complete")
    readline.set_completer(CustomCompleter().complete)

    import __main__
    if extra_vars:
        __main__.__dict__.update(
            dict((k, v) for k, v in extra_vars.iteritems() if not k.startswith('__')))

    # As completer class search complement variables in `__main__.__dict__`
    # (`self.namespace = __main__.__dict__` in 'rlcompleter.Completer.complete'),
    # and the code is executed by `exec code in self.locals`
    # (in `code.InteractiveInterpreter.runcode`),
    # `__main__.__dict__` must be used as the local variables scope,
    # not `globals()`, `locals()` or any other dict,
    # (`__main__` module is explained here: http://docs.python.org/2/library/__main__.html)
    shell = code.InteractiveConsole(__main__.__dict__)
    shell.interact()
    # code.interact(local=__main__.__dict__)


def fix_request_arguments(arguments):
    return dict((k, v[0]) for k, v in arguments.iteritems())


class LocalProxy(object):
    """Acts as a proxy for a werkzeug local.  Forwards all operations to
    a proxied object.  The only operations not supported for forwarding
    are right handed operands and any kind of assignment.
    Example usage::
        from werkzeug.local import Local
        l = Local()
        # these are proxies
        request = l('request')
        user = l('user')
        from werkzeug.local import LocalStack
        _response_local = LocalStack()
        # this is a proxy
        response = _response_local()
    Whenever something is bound to l.user / l.request the proxy objects
    will forward all operations.  If no object is bound a :exc:`RuntimeError`
    will be raised.
    To create proxies to :class:`Local` or :class:`LocalStack` objects,
    call the object as shown above.  If you want to have a proxy to an
    object looked up by a function, you can (as of Werkzeug 0.6.1) pass
    a function to the :class:`LocalProxy` constructor::
        session = LocalProxy(lambda: get_current_request().session)
    .. versionchanged:: 0.6.1
       The class can be instanciated with a callable as well now.
    """
    __slots__ = ('__local', '__dict__', '__name__')

    def __init__(self, local, name=None):
        object.__setattr__(self, '_LocalProxy__local', local)
        object.__setattr__(self, '__name__', name)

    def get_current_object(self):
        """Return the current object.  This is useful if you want the real
        object behind the proxy at a time for performance reasons or because
        you want to pass the object into a different context.
        """
        if not hasattr(self.__local, '__release_local__'):
            return self.__local()
        try:
            return getattr(self.__local, self.__name__)
        except AttributeError:
            raise RuntimeError('no object bound to %s' % self.__name__)

    @property
    def __dict__(self):
        try:
            return self.get_current_object().__dict__
        except RuntimeError:
            raise AttributeError('__dict__')

    def __repr__(self):
        try:
            obj = self.get_current_object()
        except RuntimeError:
            return '<%s unbound>' % self.__class__.__name__
        return repr(obj)

    def __bool__(self):
        try:
            return bool(self.get_current_object())
        except RuntimeError:
            return False

    def __unicode__(self):
        try:
            return unicode(self.get_current_object())
        except RuntimeError:
            return repr(self)

    def __dir__(self):
        try:
            return dir(self.get_current_object())
        except RuntimeError:
            return []

    def __getattr__(self, name):
        if name == '__members__':
            return dir(self.get_current_object())
        return getattr(self.get_current_object(), name)

    def __setitem__(self, key, value):
        self.get_current_object()[key] = value

    def __delitem__(self, key):
        del self.get_current_object()[key]

    __getslice__ = lambda x, i, j: x.get_current_object()[i:j]

    def __setslice__(self, i, j, seq):
        self.get_current_object()[i:j] = seq

    def __delslice__(self, i, j):
        del self.get_current_object()[i:j]

    __setattr__ = lambda x, n, v: setattr(x.get_current_object(), n, v)
    __delattr__ = lambda x, n: delattr(x.get_current_object(), n)
    __str__ = lambda x: str(x.get_current_object())
    __lt__ = lambda x, o: x.get_current_object() < o
    __le__ = lambda x, o: x.get_current_object() <= o
    __eq__ = lambda x, o: x.get_current_object() == o
    __ne__ = lambda x, o: x.get_current_object() != o
    __gt__ = lambda x, o: x.get_current_object() > o
    __ge__ = lambda x, o: x.get_current_object() >= o
    __cmp__ = lambda x, o: cmp(x.get_current_object(), o)
    __hash__ = lambda x: hash(x.get_current_object())
    __call__ = lambda x, *a, **kw: x.get_current_object()(*a, **kw)
    __len__ = lambda x: len(x.get_current_object())
    __getitem__ = lambda x, i: x.get_current_object()[i]
    __iter__ = lambda x: iter(x.get_current_object())
    __contains__ = lambda x, i: i in x.get_current_object()
    __add__ = lambda x, o: x.get_current_object() + o
    __sub__ = lambda x, o: x.get_current_object() - o
    __mul__ = lambda x, o: x.get_current_object() * o
    __floordiv__ = lambda x, o: x.get_current_object() // o
    __mod__ = lambda x, o: x.get_current_object() % o
    __divmod__ = lambda x, o: x.get_current_object().__divmod__(o)
    __pow__ = lambda x, o: x.get_current_object() ** o
    __lshift__ = lambda x, o: x.get_current_object() << o
    __rshift__ = lambda x, o: x.get_current_object() >> o
    __and__ = lambda x, o: x.get_current_object() & o
    __xor__ = lambda x, o: x.get_current_object() ^ o
    __or__ = lambda x, o: x.get_current_object() | o
    __div__ = lambda x, o: x.get_current_object().__div__(o)
    __truediv__ = lambda x, o: x.get_current_object().__truediv__(o)
    __neg__ = lambda x: -(x.get_current_object())
    __pos__ = lambda x: +(x.get_current_object())
    __abs__ = lambda x: abs(x.get_current_object())
    __invert__ = lambda x: ~(x.get_current_object())
    __complex__ = lambda x: complex(x.get_current_object())
    __int__ = lambda x: int(x.get_current_object())
    __long__ = lambda x: long(x.get_current_object())
    __float__ = lambda x: float(x.get_current_object())
    __oct__ = lambda x: oct(x.get_current_object())
    __hex__ = lambda x: hex(x.get_current_object())
    __index__ = lambda x: x.get_current_object().__index__()
    __coerce__ = lambda x, o: x.get_current_object().__coerce__(x, o)
    __enter__ = lambda x: x.get_current_object().__enter__()
    __exit__ = lambda x, *a, **kw: x.get_current_object().__exit__(*a, **kw)
    __radd__ = lambda x, o: o + x.get_current_object()
    __rsub__ = lambda x, o: o - x.get_current_object()
    __rmul__ = lambda x, o: o * x.get_current_object()
    __rdiv__ = lambda x, o: o / x.get_current_object()
    __rtruediv__ = lambda x, o: x.get_current_object().__rtruediv__(o)
    __rfloordiv__ = lambda x, o: o // x.get_current_object()
    __rmod__ = lambda x, o: o % x.get_current_object()
    __rdivmod__ = lambda x, o: x.get_current_object().__rdivmod__(o)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup


# Always stay on 0.9 before documentation is finished
__version__ = '0.9.5'


def get_requires():
    with open('requirements.txt', 'r') as f:
        requires = [i for i in map(lambda x: x.strip(), f.readlines()) if i]
    return requires


def get_long_description():
    try:
        with open('README.md', 'r') as f:
            return f.read()
    except IOError:
        return ''


setup(
    license='License :: OSI Approved :: MIT License',
    name='torext',
    version=__version__,
    author='reorx',
    author_email='novoreorx@gmail.com',
    url='http://github.com/reorx/torext',
    description="The missing tornado mate",
    long_description=get_long_description(),
    packages=[
        'torext',
        'torext.handlers',
        'torext.scripts',
        'torext.test',
    ],
    package_data={
        'torext': [
            'templates/project/__init__.py',
            'templates/project/app.py',
            'templates/project/settings.py',
        ]
    },
    entry_points={
        'console_scripts': [
            'torext-sketch = torext.scripts.sketch:main',
        ]
    },
    install_requires=get_requires(),
    extras_require={
        'sql': ['SQLAlchemy'],
        'gevent': ['gevent'],
    }
)
